/* -------------------------------------------------------------------------
 *
 * pgcryptokey.c
 *
 * Copyright (c) 2010-2017, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
 *		pgcryptokey.c
 *
 * -------------------------------------------------------------------------
 */
#include "postgres.h"

#include <stdio.h>

#include "storage/ipc.h"
#include "utils/guc.h"

PG_MODULE_MAGIC;

/* found via $PATH, installed in the Postgres /bin directory */
#define PG_GETKEY_EXEC	"pgcryptokey_acpass"

void		_PG_init(void);

/* GUC Variable */
static char *default_password = NULL;

/* This controls the maximum length of the read symmetric key */
static char buf[1024];

/*
 * Module Load Callback
 */
void
_PG_init(void)
{
	FILE *fp;
	
	/* Execute the command with pipe and read the standard output. */
	if ((fp = popen(PG_GETKEY_EXEC, "r")) == NULL)
	{
		fprintf(stderr, "%s: could not launch shell command\n", PG_GETKEY_EXEC);
		proc_exit(1);
	}

	/* error on a single character output, probably a newline */
	if (fgets(buf, sizeof(buf), fp) == NULL || strlen(buf) <= 1)
	{
		fprintf(stderr, "%s: no password output\n", PG_GETKEY_EXEC);
		(void) pclose(fp);
		proc_exit(1);
	}

	if (strlen(buf) == sizeof(buf) - 1)
	{
		fprintf(stderr, "%s: password is too long\n", PG_GETKEY_EXEC);
		(void) pclose(fp);
		proc_exit(1);
	}

	/* trim off trailing newline */
	if (buf[strlen(buf) - 1] == '\n')
		buf[strlen(buf) - 1] = '\0';

	if (pclose(fp) != 0)
	{
		fprintf(stderr, "%s: could not close shell command\n", PG_GETKEY_EXEC);
		proc_exit(1);
	}

	/* Define custom GUC variables */
	DefineCustomStringVariable("pgcryptokey.default_password",
							"pgcryptokey default password",
							NULL,
							&default_password,
							buf,
							PGC_POSTMASTER,
							0,
							NULL,
							NULL,
							NULL);
}
